<!DOCTYPE html>
<html>
<head>
  {% block head %}
    <title>Appyter</title>
  {% endblock %}
  {% block style %}
    <link rel="stylesheet" href="{{ url_for('static', filename='css/index.css') }}" />
  {% endblock %}
  {% block script_head %}
    <script src="{{ url_for('static', filename='js/lib/require.js') }}"></script>
    <script>
      require.config({
        paths: {
          'jquery': {{ url_for('static', filename='js/lib/jquery')|jsonify }},
        },
        waitSeconds: 200,
      })
    </script>
    <script>
      window._config = {
        DEBUG: {{ _config.DEBUG|jsonify }},
        PROFILE: {{ _config.PROFILE|jsonify }},
        EXTRAS: {{ _config.EXTRAS|jsonify }},
        PREFIX: {{ _config.PREFIX|jsonify }},
        STATIC: {{ url_for('static', filename='')|jsonify }},
        ORIGIN: (window.location.origin + {{ _config.PREFIX|jsonify }}),
        {% if 'catalog-integration' in _config.EXTRAS %}
        CATALOG_ORIGIN: window.location.origin,
        HINTS: {{ _config.HINTS|jsonify }},
        {% endif %}
      }
    </script>
  {% endblock %}
</head>
<body>
  {% block header %}{% endblock %}
  {% block body %}{% endblock %}
  {% block footer %}{% endblock %}
  {% block script %}
    <script>
      require([{{ url_for('static', filename='js/index.js')|jsonify }}], function (appyter) {
        appyter.init(window)
      })
    </script>
  {% endblock %}
</body>
</html>