<div id={{ this.args.name|jsonify }} class="svelte loading">
  {% include "fragments/loader.j2" %}
  <script type="application/json">{{ dict(args=this.args, js_url=this.js_url)|jsonify }}</script>
</div>