"use strict";(self.webpackChunkappyter_js=self.webpackChunkappyter_js||[]).push([[59],{2059:(s,e,n)=>{n.r(e),n.d(e,{default:()=>o});var t=n(7418);function u(s){let e,n;const u=s[1].default,c=(0,t.Of3)(u,s,s[0],null);return{c(){e=(0,t.ND4)("div"),c&&c.c(),(0,t.CFu)(e,"id","notebook"),(0,t.CFu)(e,"class","border-box-sizing")},m(s,u){(0,t.Yry)(s,e,u),c&&c.m(e,null),n=!0},p(s,[e]){c&&c.p&&(!n||1&e)&&(0,t.nkG)(c,u,s,s[0],n?(0,t.sWk)(u,s[0],e,null):(0,t.i32)(s[0]),null)},i(s){n||((0,t.c7F)(c,s),n=!0)},o(s){(0,t.Tn8)(c,s),n=!1},d(s){s&&(0,t.YoD)(e),c&&c.d(s)}}}function c(s,e,n){let{$$slots:t={},$$scope:u}=e;return s.$$set=s=>{"$$scope"in s&&n(0,u=s.$$scope)},[u,t]}n(5866);class l extends t.r7T{constructor(s){super(),(0,t.TsN)(this,s,c,u,t.jXN,{})}}const o=l}}]);